'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _addInstrumentation;

exports.default = function () {
  return {
    visitor: {
      Program: function (path, state) {
        var scope = path.scope;
        var _state$file = state.file;
        var ast = _state$file.ast;
        var filename = _state$file.opts.filename;
        var _state$opts$collector = state.opts.collectorModuleId;
        var collectorModuleId = _state$opts$collector === undefined ? 'copenhagen/collector' : _state$opts$collector;


        var HASH = t.stringLiteral((0, _hashAst2.default)(filename, ast));

        var COLLECTOR = t.identifier(scope.generateUid('collector'));
        var COUNT_S = t.identifier(scope.generateUid('s'));
        var COUNT_F = t.identifier(scope.generateUid('f'));

        var statementId = 0;
        var fnId = 0;
        path.traverse(addInstrumentation, {
          countStatement: function () {
            return buildStatementCount({
              COUNT_S: COUNT_S,
              ID: t.numericLiteral(++statementId)
            });
          },
          countInvocation: function () {
            return buildInvocationCount({
              COUNT_F: COUNT_F,
              ID: t.numericLiteral(++fnId)
            });
          }
        });

        var CREATE_COLLECTOR = state.addImport(collectorModuleId, 'createCollector');
        path.unshiftContainer('body', buildCreateCollector({
          COLLECTOR: COLLECTOR,
          COUNT_F: COUNT_F,
          COUNT_S: COUNT_S,
          CREATE_COLLECTOR: CREATE_COLLECTOR,
          HASH: HASH
        }));
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _hashAst = require('../lib/hash-ast');

var _hashAst2 = _interopRequireDefault(_hashAst);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var buildCreateCollector = (0, _babelTemplate2.default)('\n  var COLLECTOR = CREATE_COLLECTOR(HASH);\n  var COUNT_S = COLLECTOR.s;\n  var COUNT_F = COLLECTOR.f;\n');

var buildStatementCount = (0, _babelTemplate2.default)('COUNT_S(ID)');
var buildInvocationCount = (0, _babelTemplate2.default)('COUNT_F(ID)');

var statementMatcher = '\n  AssignmentExpression\n  ReturnStatement\n  VariableDeclaration\n'.trim().split(/\s+/).join('|');

var addInstrumentation = (_addInstrumentation = {}, _defineProperty(_addInstrumentation, statementMatcher, function (path) {
  path.insertBefore(this.countStatement());
}), _defineProperty(_addInstrumentation, 'Function', function (path) {
  path.node.body.body.unshift(this.countInvocation());
}), _addInstrumentation);
//# sourceMappingURL=add-instrumentation.js.map