'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _addToMaps;

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      Program: function (path, state) {
        if (!path.node.loc) return;

        var _state$file = state.file;
        var ast = _state$file.ast;
        var filename = _state$file.opts.filename;


        var PATH = t.stringLiteral(filename);
        var HASH = t.stringLiteral((0, _hashAst2.default)(filename, ast));

        var ADD_FUNCTIONS = [];
        var ADD_STATEMENTS = [];

        var anonCount = 0;
        var fnId = 0;
        var statementId = 0;
        path.traverse(addToMaps, {
          addFn: function (name, start, end) {
            ADD_FUNCTIONS.push.apply(ADD_FUNCTIONS, _toConsumableArray(fnMapAdditionBuilder({
              ID: t.numericLiteral(++fnId),
              NAME: t.stringLiteral(name || '(anonymous_' + ++anonCount + ')'),
              LINE: t.numericLiteral(start.line),
              LOC: locBuilder({
                START_LINE: t.numericLiteral(start.line),
                START_COLUMN: t.numericLiteral(start.column),
                END_LINE: t.numericLiteral(end.line),
                END_COLUMN: t.numericLiteral(end.column)
              }).argument
            })));
          },
          addStatement: function (start, end) {
            ADD_STATEMENTS.push.apply(ADD_STATEMENTS, _toConsumableArray(statementMapAdditionBuilder({
              ID: t.numericLiteral(++statementId),
              LOC: locBuilder({
                START_LINE: t.numericLiteral(start.line),
                START_COLUMN: t.numericLiteral(start.column),
                END_LINE: t.numericLiteral(end.line),
                END_COLUMN: t.numericLiteral(end.column)
              }).argument
            })));
          }
        });

        var GET_INITIAL_COVERAGE = t.identifier('getInitialCoverage');
        path.replaceWith(t.program([programBuilder({
          ADD_FUNCTIONS: ADD_FUNCTIONS,
          ADD_STATEMENTS: ADD_STATEMENTS,
          GET_INITIAL_COVERAGE: GET_INITIAL_COVERAGE,
          PATH: PATH
        }), t.exportNamedDeclaration(null, [t.exportSpecifier(GET_INITIAL_COVERAGE, GET_INITIAL_COVERAGE)]), t.exportNamedDeclaration(t.variableDeclaration('var', [t.variableDeclarator(t.identifier('hash'), HASH)]), [])], [t.Directive(t.DirectiveLiteral('use strict'))]));
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _hashAst = require('../lib/hash-ast');

var _hashAst2 = _interopRequireDefault(_hashAst);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var programBuilder = (0, _babelTemplate2.default)('\n  function GET_INITIAL_COVERAGE () {\n    var s = {};\n    var b = {};\n    var f = {};\n    var fnMap = {};\n    var statementMap = {};\n    var branchMap = {};\n    var report = {\n      path: PATH,\n      s: s,\n      b: b,\n      f: f,\n      fnMap: fnMap,\n      statementMap: statementMap,\n      branchMap: branchMap\n    };\n    ADD_FUNCTIONS\n    ADD_STATEMENTS\n    return report;\n  }\n');

var locBuilder = (0, _babelTemplate2.default)('\n  return {\n    start: {\n      line: START_LINE,\n      column: START_COLUMN\n    },\n    end: {\n      line: END_LINE,\n      column: END_COLUMN\n    }\n  }\n');

var fnMapAdditionBuilder = (0, _babelTemplate2.default)('\n  f[ID] = 0\n  fnMap[ID] = {\n    name: NAME,\n    line: LINE,\n    loc: LOC\n  }\n');

var statementMapAdditionBuilder = (0, _babelTemplate2.default)('\n  s[ID] = 0\n  statementMap[ID] = LOC\n');

var statementMatcher = '\n  AssignmentExpression\n  ReturnStatement\n  VariableDeclaration\n'.trim().split(/\s+/).join('|');

var addToMaps = (_addToMaps = {}, _defineProperty(_addToMaps, statementMatcher, function (path) {
  var _path$node$loc = path.node.loc;
  var start = _path$node$loc.start;
  var end = _path$node$loc.end;

  this.addStatement(start, end);
}), _defineProperty(_addToMaps, 'FunctionDeclaration', function (path) {
  var _path$node = path.node;
  var name = _path$node.id.name;
  var _path$node$loc2 = _path$node.loc;
  var start = _path$node$loc2.start;
  var definitionEnd = _path$node$loc2.end;
  var end = _path$node.body.loc.start;


  this.addFn(name, start, end);
  this.addStatement(start, definitionEnd);
}), _defineProperty(_addToMaps, 'FunctionExpression', function (path) {
  var _path$node2 = path.node;
  var start = _path$node2.loc.start;
  var end = _path$node2.body.loc.start;


  this.addFn(null, start, end);
}), _addToMaps);
//# sourceMappingURL=generate-reporting-code.js.map