'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addCounts = addCounts;
exports.collectUsageData = collectUsageData;

var _csvParser = require('csv-parser');

var _csvParser2 = _interopRequireDefault(_csvParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _Promise = typeof Promise === 'undefined' ? require('es6-promise').Promise : Promise;

function addCounts(coverage, usageData) {
  // for/of would be the preferrable syntax but that requires an inlined
  // iterator implementation to support older Node.js versions. An old-fashioned
  // loop does the job as well.
  for (var i = 0; i < usageData.length; i++) {
    var _usageData$i = usageData[i];
    var type = _usageData$i.type;
    var id = _usageData$i.id;

    coverage[type][id]++;
  }
}

function collectStream(result, stream) {
  return new _Promise(function (resolve, reject) {
    var parser = (0, _csvParser2.default)(['hash', 'type', 'id']).on('data', function (_ref) {
      var hash = _ref.hash;
      var type = _ref.type;
      var id = _ref.id;

      var usage = result[hash] || (result[hash] = []);
      usage.push({ id: id, type: type });
    }).on('end', resolve);

    stream.once('error', function (err) {
      stream.unpipe(parser);
      reject(err);
    }).pipe(parser);
  });
}

function collectUsageData(streams) {
  if (!Array.isArray(streams)) {
    streams = [streams];
  }

  var result = Object.create(null);
  return Promise.all(streams.map(function (stream) {
    return collectStream(result, stream);
  })).then(function () {
    return result;
  });
}
//# sourceMappingURL=coverage.js.map